@@ -1,4 +1,4 @@
-# Copyright 2004-2021 Bright Computing Holding BV
+# Copyright 2004-2022 Bright Computing Holding BV
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -90,21 +90,19 @@ def create_vpc(cluster):
 
     This VPC contains a single head node, two subnets, an internal gateway and one routing table.
     """
-    vpc_name = cluster.name
     ec2 = cluster.ec2
 
-    head_node_hostname = vpc_name
-    vpc_name = get_vpc_name_for_cluster(vpc_name)
+    head_node_hostname = cluster.name
+    vpc_name = get_vpc_name_for_cluster(cluster.name)
 
     if _vpc_with_name_exists(vpc_name, ec2):
         raise UserReportableException("VPC for this cluster already exists")
 
     _report_progress(2, "Setting up VPC")
 
-    cluster.vpc = _create_vpc(vpc_name, ec2)
+    cluster.vpc = _create_vpc(cluster.name, vpc_name, ec2)
     cluster.vpc.name = vpc_name
     cluster.vpc.cluster_name = cluster.name
-    _apply_bright_tags(cluster.vpc, cluster.vpc)
 
     if config["configure_igw"]:
         internet_gateway = _create_internet_gateway(cluster.vpc, ec2)
@@ -145,12 +143,16 @@ def _retry_not_found(func, *args, **kwargs):
     return retry_wrapper(func, *args, **kwargs)
 
 
-def _create_vpc(vpc_name, ec2):
+def _create_vpc(cluster_name, vpc_name, ec2):
     _report_progress(5, "Creating VPC for %s in %s..." % (vpc_name, config["aws_region"]))
 
-    vpc = ec2.create_vpc(CidrBlock=str(config["vpc_cidr"]))
-    # Make sure the VPC has finished creation before setting tags on it.
-    # See https://docs.aws.amazon.com/AWSEC2/latest/APIReference/query-api-troubleshooting.html#eventual-consistency
+    # https://aws.amazon.com/about-aws/whats-new/2020/07/amazon-vpc-resources-support-tag-on-create/
+    tags = _construct_bright_tags(cluster_name, vpc_name)
+    kwargs = {"CidrBlock": str(config["vpc_cidr"]),
+              "TagSpecifications": [{"ResourceType": "vpc", "Tags": tags}]}
+
+    vpc = ec2.create_vpc(**kwargs)
+    # Make sure the VPC has finished creation before using it.
     vpc.wait_until_exists()
     _retry_not_found(vpc.wait_until_available)
     log.debug("VPC was successfully created, id: %s", vpc.id)
@@ -161,11 +163,14 @@ def _create_vpc(vpc_name, ec2):
 def _create_internet_gateway(vpc, ec2):
     _report_progress(10, "Adding internet connectivity...")
 
-    internet_gateway = ec2.create_internet_gateway()
+    internet_gateway_name = vpc.cluster_name
+    tags = _construct_bright_tags(vpc.cluster_name, internet_gateway_name)
+    kwargs = {"TagSpecifications": [{"ResourceType": "internet-gateway", "Tags": tags}]}
+
+    internet_gateway = ec2.create_internet_gateway(**kwargs)
     time.sleep(0.5)
 
-    internet_gateway.name = vpc.cluster_name
-    _apply_bright_tags(vpc, internet_gateway)
+    internet_gateway.name = internet_gateway_name
 
     vpc.attach_internet_gateway(InternetGatewayId=internet_gateway.id)
     return internet_gateway
@@ -173,14 +178,13 @@ def _create_internet_gateway(vpc, ec2):
 
 def _create_public_subnet(vpc, ec2):
     _report_progress(12, "Creating public subnet...")
-
     kwargs = {"CidrBlock": str(config["public_subnet_cidr"])}
 
     if config["aws_availability_zone"]:
         kwargs["AvailabilityZone"] = config["aws_availability_zone"]
 
     public_subnet_name = vpc.cluster_name + " public"
-    tags = _construct_bright_tags(vpc, public_subnet_name, BCM_TYPE_HEAD_NODE)
+    tags = _construct_bright_tags(vpc.cluster_name, public_subnet_name, BCM_TYPE_HEAD_NODE)
     kwargs["TagSpecifications"] = [{"ResourceType": "subnet", "Tags": tags}]
 
     public_subnet = vpc.create_subnet(**kwargs)
@@ -199,7 +203,7 @@ def _create_private_subnet(vpc, ec2):
         kwargs["AvailabilityZone"] = config["aws_availability_zone"]
 
     private_subnet_name = vpc.cluster_name + " private"
-    tags = _construct_bright_tags(vpc, private_subnet_name, BCM_TYPE_HEAD_NODE)
+    tags = _construct_bright_tags(vpc.cluster_name, private_subnet_name, BCM_TYPE_HEAD_NODE)
     kwargs["TagSpecifications"] = [{"ResourceType": "subnet", "Tags": tags}]
 
     private_subnet = vpc.create_subnet(**kwargs)
@@ -226,9 +230,13 @@ def _set_default_gateway_vpc(vpc, internet_gateway):
     reraise=True
 )
 def _set_default_gateway_subnet(vpc, subnet, internet_gateway):
-    route_table = vpc.create_route_table()
-    route_table.name = subnet.name + " route table"
-    _apply_bright_tags(vpc, route_table)
+    route_table_name = subnet.name + " route table"
+    tags = _construct_bright_tags(vpc.cluster_name, route_table_name)
+    rt_kwargs = {"TagSpecifications": [{"ResourceType": "route-table", "Tags": tags}]}
+
+    route_table = vpc.create_route_table(**rt_kwargs)
+
+    route_table.name = route_table_name
 
     route_table.associate_with_subnet(SubnetId=subnet.id)
 
@@ -353,7 +361,7 @@ def _create_head_node(vpc, hostname, public_subnet, head_node_image, head_node_s
     log.info("Creating the head node VM instance")
 
     head_node_instance_name = f"{vpc.cluster_name} (Bright COD-AWS {config['version']} head node)"
-    tags = _construct_bright_tags(vpc, head_node_instance_name, BCM_TYPE_HEAD_NODE)
+    tags = _construct_bright_tags(vpc.cluster_name, head_node_instance_name, BCM_TYPE_HEAD_NODE)
     kwargs["TagSpecifications"] = [{"ResourceType": "instance", "Tags": tags}]
 
     instances = public_subnet.create_instances(**kwargs)
@@ -447,11 +455,11 @@ def _find_main_route_table(vpc):
     return route_tables[0]
 
 
-def _construct_bright_tags(vpc, obj_name, bcm_type=None):
+def _construct_bright_tags(cluster_name, obj_name, bcm_type=None):
     tags = {
         "BCM Created at": datetime.utcnow().isoformat() + "Z",  # fix missing timezone
         "BCM Created by": utils.get_user_at_fqdn_hostname(),
-        "BCM Cluster": vpc.cluster_name,
+        "BCM Cluster": cluster_name,
         "BCM Bursting": "on-demand",
         # This one has to follow a slightly different naming convention (i.e. no "BCM"
         # prefix), because this tag has a special meaning for AWS.
@@ -465,11 +473,6 @@ def _construct_bright_tags(vpc, obj_name, bcm_type=None):
     return tags
 
 
-def _apply_bright_tags(vpc, obj, bcm_type=None):
-    tags = _construct_bright_tags(vpc, obj.name, bcm_type)
-    _retry_not_found(obj.create_tags, Tags=tags)
-
-
 def get_aws_tag(tags: List[Dict[str, str]], name: str, default: str = None) -> str:
     """
     For some mysterious reason, boto3 stores tags as a list of dictionaries:
